@@ -6,6 +6,7 @@ import dbt.project
 import dbt.utils
 import dbt.include
 import dbt.wrapper
+import dbt.tracking
 
 from dbt.model import Model
 from dbt.utils import This, Var, is_enabled, get_materialization, NodeType, \
@@ -38,6 +39,7 @@ def print_compile_stats(stats):
         NodeType.Archive: 'archives',
         NodeType.Analysis: 'analyses',
         NodeType.Macro: 'macros',
+        NodeType.Operation: 'operations',
     }
 
     results = {
@@ -46,6 +48,7 @@ def print_compile_stats(stats):
         NodeType.Archive: 0,
         NodeType.Analysis: 0,
         NodeType.Macro: 0,
+        NodeType.Operation: 0,
     }
 
     results.update(stats)
@@ -53,7 +56,7 @@ def print_compile_stats(stats):
     stat_line = ", ".join(
         ["{} {}".format(ct, names.get(t)) for t, ct in results.items()])
 
-    logger.info("Compiled {}".format(stat_line))
+    logger.info("Found {}".format(stat_line))
 
 
 def prepend_ctes(model, flat_graph):
@@ -73,7 +76,7 @@ def recursively_prepend_ctes(model, flat_graph):
     if model.get('all_ctes_injected') is True:
         return (model, model.get('extra_ctes').keys(), flat_graph)
 
-    for cte_id in model.get('extra_ctes').keys():
+    for cte_id in model.get('extra_ctes', {}).keys():
         cte_to_add = flat_graph.get('nodes').get(cte_id)
         cte_to_add, new_prepend_ctes, flat_graph = recursively_prepend_ctes(
             cte_to_add, flat_graph)
@@ -153,18 +156,13 @@ class Compiler(object):
         self.parsed_models = None
 
     def initialize(self):
-        if not os.path.exists(self.project['target-path']):
-            os.makedirs(self.project['target-path'])
-
-        if not os.path.exists(self.project['modules-path']):
-            os.makedirs(self.project['modules-path'])
+        dbt.clients.system.make_directory(self.project['target-path'])
+        dbt.clients.system.make_directory(self.project['modules-path'])
 
     def __write(self, build_filepath, payload):
         target_path = os.path.join(self.project['target-path'], build_filepath)
 
-        if not os.path.exists(os.path.dirname(target_path)):
-            os.makedirs(os.path.dirname(target_path))
-
+        dbt.clients.system.make_directory(os.path.dirname(target_path))
         dbt.compat.write_file(target_path, payload)
 
         return target_path
@@ -175,7 +173,7 @@ class Compiler(object):
 
         return do_config
 
-    def __ref(self, ctx, model, all_models):
+    def __ref(self, ctx, model, flat_graph, current_project):
         schema = ctx.get('env', {}).get('schema')
 
         def do_ref(*args):
@@ -189,10 +187,12 @@ class Compiler(object):
             else:
                 dbt.exceptions.ref_invalid_args(model, args)
 
-            target_model = dbt.utils.find_model_by_name(
-                all_models,
+            target_model = dbt.parser.resolve_ref(
+                flat_graph,
                 target_model_name,
-                target_model_package)
+                target_model_package,
+                current_project,
+                model.get('package_name'))
 
             if target_model is None:
                 dbt.exceptions.ref_target_not_found(
@@ -223,7 +223,8 @@ class Compiler(object):
         wrapper = dbt.wrapper.DatabaseWrapper(model, adapter, profile)
 
         # built-ins
-        context['ref'] = self.__ref(context, model, flat_graph)
+        context['ref'] = self.__ref(context, model, flat_graph,
+                                    self.project.cfg.get('name'))
         context['config'] = self.__model_config(model)
         context['this'] = This(
             context['env']['schema'],
@@ -237,8 +238,8 @@ class Compiler(object):
 
         context.update(wrapper.get_context_functions())
 
-        context['run_started_at'] = '{{ run_started_at }}'
-        context['invocation_id'] = '{{ invocation_id }}'
+        context['run_started_at'] = dbt.tracking.active_user.run_started_at
+        context['invocation_id'] = dbt.tracking.active_user.invocation_id
         context['sql_now'] = adapter.date_function()
 
         for unique_id, macro in flat_graph.get('macros').items():
@@ -282,7 +283,8 @@ class Compiler(object):
         injected_node, _ = prepend_ctes(compiled_node, flat_graph)
 
         if compiled_node.get('resource_type') in [NodeType.Test,
-                                                  NodeType.Analysis]:
+                                                  NodeType.Analysis,
+                                                  NodeType.Operation]:
             # data tests get wrapped in count(*)
             # TODO : move this somewhere more reasonable
             if 'data' in injected_node['tags'] and \
@@ -352,7 +354,7 @@ class Compiler(object):
     def link_graph(self, linker, flat_graph):
         linked_graph = {
             'nodes': {},
-            'macros': flat_graph.get('macros'),
+            'macros': flat_graph.get('macros')
         }
 
         for name, node in flat_graph.get('nodes').items():
@@ -470,6 +472,8 @@ class Compiler(object):
         all_nodes.update(
             dbt.parser.parse_archives_from_projects(root_project,
                                                     all_projects))
+        all_nodes.update(
+            dbt.parser.load_and_parse_run_hooks(root_project, all_projects))
 
         return all_nodes
 
@@ -480,6 +484,7 @@ class Compiler(object):
         all_projects = self.get_all_projects()
 
         all_macros = self.load_all_macros(root_project, all_projects)
+
         all_nodes = self.load_all_nodes(root_project, all_projects)
 
         flat_graph = {
@@ -487,7 +492,8 @@ class Compiler(object):
             'macros': all_macros
         }
 
-        flat_graph = dbt.parser.process_refs(flat_graph)
+        flat_graph = dbt.parser.process_refs(flat_graph,
+                                             root_project.get('name'))
 
         linked_graph = self.link_graph(linker, flat_graph)
 
@@ -499,6 +505,7 @@ class Compiler(object):
         for node_name, node in linked_graph.get('macros').items():
             stats[node.get('resource_type')] += 1
 
+        self.write_graph_file(linker)
         print_compile_stats(stats)
 
         return linked_graph, linker
