@@ -1,6 +1,15 @@
 import networkx as nx
 from collections import defaultdict
 
+import dbt.utils
+
+
+def from_file(graph_file):
+    linker = Linker()
+    linker.read_graph(graph_file)
+
+    return linker
+
 
 class Linker(object):
     def __init__(self, data=None):
@@ -9,6 +18,9 @@ class Linker(object):
         self.graph = nx.DiGraph(**data)
         self.cte_map = defaultdict(set)
 
+    def edges(self):
+        return self.graph.edges()
+
     def nodes(self):
         return self.graph.nodes()
 
@@ -18,6 +30,17 @@ class Linker(object):
     def get_node(self, node):
         return self.graph.node[node]
 
+    def find_cycles(self):
+        try:
+            cycles = nx.algorithms.find_cycle(self.graph)
+        except nx.exception.NetworkXNoCycle:
+            return None
+
+        if cycles:
+            return " --> ".join([".".join(node) for node in cycles])
+
+        return None
+
     def as_topological_ordering(self, limit_to=None):
         try:
             return nx.topological_sort(self.graph, nbunch=limit_to)
@@ -37,31 +60,37 @@ class Linker(object):
                 "{}".format(cycle)
             )
 
-    def as_dependency_list(self, limit_to=None):
+    def as_dependency_list(self, limit_to=None, ephemeral_only=False):
         """returns a list of list of nodes, eg. [[0,1], [2], [4,5,6]]. Each
         element contains nodes whose dependenices are subsumed by the union of
         all lists before it. In this way, all nodes in list `i` can be run
         simultaneously assuming that all lists before list `i` have been
         completed"""
 
-        if limit_to is None:
-            graph_nodes = set(self.graph.nodes())
-        else:
-            graph_nodes = set()
-            for node in limit_to:
-                graph_nodes.add(node)
-                if node in self.graph:
-                    graph_nodes.update(nx.descendants(self.graph, node))
-                else:
-                    raise RuntimeError(
-                        "Couldn't find model '{}' -- does it exist or is "
-                        "it disabled?".format(node)
-                    )
-
         depth_nodes = defaultdict(list)
 
+        if limit_to is None:
+            graph_nodes = self.graph.nodes()
+        else:
+            graph_nodes = limit_to
+
         for node in graph_nodes:
-            num_ancestors = len(nx.ancestors(self.graph, node))
+            if node not in self.graph:
+                raise RuntimeError(
+                    "Couldn't find model '{}' -- does it exist or is "
+                    "it disabled?".format(node)
+                )
+
+            num_ancestors = len([
+                ancestor for ancestor in
+                nx.ancestors(self.graph, node)
+                if (dbt.utils.is_blocking_dependency(
+                        self.get_node(ancestor)) and
+                    (ephemeral_only is False or
+                     dbt.utils.get_materialization(
+                         self.get_node(ancestor)) == 'ephemeral'))
+            ])
+
             depth_nodes[num_ancestors].append(node)
 
         dependency_list = []
@@ -94,7 +123,7 @@ class Linker(object):
         self.graph.add_node(node, data)
 
     def write_graph(self, outfile):
-        nx.write_yaml(self.graph, outfile)
+        nx.write_gpickle(self.graph, outfile)
 
     def read_graph(self, infile):
-        self.graph = nx.read_yaml(infile)
+        self.graph = nx.read_gpickle(infile)
